/* This file is ifile.h - main header file containing all global variables,
   function prototypes and structure declarations */

/* ifile - intelligent mail filter for EXMH/MH
   Copyright (C) 1997  Jason Rennie <jr6b+@andrew.cmu.edu>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
   */

#ifndef __IFILE_H_
#define __IFILE_H_

#define IFILE_MAJOR_VERSION_NUMBER 0
#define IFILE_MINOR_VERSION_NUMBER 4
#define IFILE_TRIFLING_VERSION_NUMBER 5

#define FALSE 0
#define TRUE 1

#ifndef MAX_STR_LEN
#define MAX_STR_LEN 2048
#endif

/* the following constants will be modified when you run the install.perl
   script */

#ifndef MH_REFILE
#define MH_REFILE "/usr/bin/mh/refile.bak"
#endif
#ifndef MH_RCVSTORE
#define MH_RCVSTORE "/usr/lib/mh/rcvstore"
#endif
#ifndef IFILE_ACCURACY
#define IFILE_ACCURACY ".idata_accuracy"
#endif
#ifndef IFILE_DATA
#define IFILE_DATA ".idata"
#endif
#ifndef MH_PROFILE
#define MH_PROFILE ".mh_profile"
#endif
#ifndef IFILE_QUEUE
#define IFILE_QUEUE ".idata.queue"
#endif

#ifndef CLOCKS_PER_SECOND
  #ifdef CLOCKS_PER_SEC
  #define CLOCKS_PER_SECOND CLOCKS_PER_SEC
  #else
    #ifdef CLK_TCK
    #define CLOCKS_PER_SECOND CLK_TCK
    #else
    #define CLOCKS_PER_SECOND 1000
    #endif
  #endif
#endif

#include "extendable_array.h"   /* extendable array data structure plus */
				/* #defined functions for manipulation */
#include "assoc_array_int.h"

/* Progress and error reporting.  Setting in error.c. */
/* Adapted from libbow - written by Andrew Kachites McCallum */
enum ifile_verbosity_levels {
  ifile_silent = 0, /* only fatal errors */
  ifile_quiet,      /* only warnings and errors */
  ifile_progress,   /* enough lines to show progress */
  ifile_verbose,    /* lots of status info */
  ifile_debug       /* everything (and then some) */
};

/* A linked list of digits */
struct linked_list {
  struct linked_list * next;
  int digit;
};

typedef struct linked_list linked_list;

struct _mh_profile_struct
{
  char * draft_folder;     /* folder not to filter mail into */
  char * mail_path;        /* directory under which mail folders are stored */
  char * context_file;     /* stores name of current folder */
  char * inbox_folder;     /* default mailbox where mail is filtered */
};

typedef struct _mh_profile_struct mh_profile_struct;

/* not yet in use */
struct _user_profile_struct
{
  char * home_path;          /* user's home directory */
  char * source_folder;      /* origination folder for refiling */
  char * source_folder_path; /* full path for source folder */
  char * dest_folder;        /* destination folder for refiling */
  char * dest_folder_path;   /* full path for destination folder */
  char ** source_files;      /* full path for source file - alternate way to
				specify source */
};

typedef struct _user_profile_struct user_profile_struct;

/* not yet in use */
struct _msg_word
{
  char * word;
  long freq;
};

typedef struct _msg_word msg_word;

char * ifile_sprintf (char * format, ...);
char * ifile_cats (int num_strings, ...);
char * itoa (int number);
char * readline (char * string, int max_len, FILE * stream);

int ifile_rcvstore (extendable_array lines, char *mbox);
int ifile_exe (char * file);
int ifile_mh_refile (int argc, char **argv);
void ifile_free (void * var);
void ifile_update_accuracy (char * acc_file, int filters, int refiles);
mh_profile_struct ifile_read_profile (char * home_dir);

int ifile_valid_line (char * line);
void ifile_read_message (FILE * FP);

int ifile_read_category_doc_counts (FILE * DATA);
int ifile_read_category_word_counts (FILE * DATA);
int ifile_read_category_names (FILE * DATA);

int ifile_rate_categories (void);
void ifile_write_queue_entry (FILE * DATA, char * dest_folder);
int ifile_update_categories (char * folders_file);

int ifile_write_idata (char * idata_file);
int ifile_write_category_names (FILE * DATA);
void ifile_write_category_word_counts (FILE * DATA);
void ifile_write_category_doc_counts (FILE * DATA);
int ifile_write_word_frequencies (FILE * DATA);
int ifile_write_frequency_line (FILE * DATA, char * word);

#endif

